@@ -2,7 +2,7 @@ import xcffib
 import struct
 import six
 MAJOR_VERSION = 1
-MINOR_VERSION = 4
+MINOR_VERSION = 5
 key = xcffib.ExtensionKey("RANDR")
 _events = {}
 _errors = {}
@@ -368,29 +368,6 @@ class Transform:
     ScaleUp = 1 << 1
     ScaleDown = 1 << 2
     Projective = 1 << 3
-class GetCrtcTransformReply(xcffib.Reply):
-    def __init__(self, unpacker):
-        if isinstance(unpacker, xcffib.Protobj):
-            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
-        xcffib.Reply.__init__(self, unpacker)
-        base = unpacker.offset
-        unpacker.unpack("xx2x4x")
-        self.pending_transform = render.TRANSFORM(unpacker)
-        self.has_transforms, = unpacker.unpack("B3x")
-        unpacker.pad(render.TRANSFORM)
-        self.current_transform = render.TRANSFORM(unpacker)
-        self.pending_len, self.pending_nparams, self.current_len, self.current_nparams = unpacker.unpack("4xHHHH")
-        unpacker.pad("c")
-        self.pending_filter_name = xcffib.List(unpacker, "c", self.pending_len)
-        unpacker.pad("i")
-        self.pending_params = xcffib.List(unpacker, "i", self.pending_nparams)
-        unpacker.pad("c")
-        self.current_filter_name = xcffib.List(unpacker, "c", self.current_len)
-        unpacker.pad("i")
-        self.current_params = xcffib.List(unpacker, "i", self.current_nparams)
-        self.bufsize = unpacker.offset - base
-class GetCrtcTransformCookie(xcffib.Cookie):
-    reply_type = GetCrtcTransformReply
 class GetPanningReply(xcffib.Reply):
     def __init__(self, unpacker):
         if isinstance(unpacker, xcffib.Protobj):
@@ -703,6 +680,46 @@ class NotifyEvent(xcffib.Event):
         self.u = u
         return self
 _events[1] = NotifyEvent
+class MonitorInfo(xcffib.Struct):
+    def __init__(self, unpacker):
+        if isinstance(unpacker, xcffib.Protobj):
+            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
+        xcffib.Struct.__init__(self, unpacker)
+        base = unpacker.offset
+        self.name, self.primary, self.automatic, self.nOutput, self.x, self.y, self.width, self.height, self.width_in_millimeters, self.height_in_millimeters = unpacker.unpack("IBBHhhHHII")
+        self.outputs = xcffib.List(unpacker, "I", self.nOutput)
+        self.bufsize = unpacker.offset - base
+    def pack(self):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=IBBHhhHHII", self.name, self.primary, self.automatic, self.nOutput, self.x, self.y, self.width, self.height, self.width_in_millimeters, self.height_in_millimeters))
+        buf.write(xcffib.pack_list(self.outputs, "I"))
+        return buf.getvalue()
+    @classmethod
+    def synthetic(cls, name, primary, automatic, nOutput, x, y, width, height, width_in_millimeters, height_in_millimeters, outputs):
+        self = cls.__new__(cls)
+        self.name = name
+        self.primary = primary
+        self.automatic = automatic
+        self.nOutput = nOutput
+        self.x = x
+        self.y = y
+        self.width = width
+        self.height = height
+        self.width_in_millimeters = width_in_millimeters
+        self.height_in_millimeters = height_in_millimeters
+        self.outputs = outputs
+        return self
+class GetMonitorsReply(xcffib.Reply):
+    def __init__(self, unpacker):
+        if isinstance(unpacker, xcffib.Protobj):
+            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
+        xcffib.Reply.__init__(self, unpacker)
+        base = unpacker.offset
+        self.timestamp, self.nMonitors, self.nOutputs = unpacker.unpack("xx2x4xIII12x")
+        self.monitors = xcffib.List(unpacker, MonitorInfo, self.nMonitors)
+        self.bufsize = unpacker.offset - base
+class GetMonitorsCookie(xcffib.Cookie):
+    reply_type = GetMonitorsReply
 class randrExtension(xcffib.Extension):
     def QueryVersion(self, major_version, minor_version, is_checked=True):
         buf = six.BytesIO()
@@ -747,7 +764,6 @@ class randrExtension(xcffib.Extension):
     def ConfigureOutputProperty(self, output, property, pending, range, values_len, values, is_checked=False):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xIIBB2x", output, property, pending, range))
-        buf.write("")
         buf.write(xcffib.pack_list(values, "i"))
         return self.send_request(12, buf, is_checked=is_checked)
     def ChangeOutputProperty(self, output, property, type, format, mode, num_units, data, is_checked=False):
@@ -767,7 +783,6 @@ class randrExtension(xcffib.Extension):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xI", window))
         buf.write(mode_info.pack() if hasattr(mode_info, "pack") else ModeInfo.synthetic(*mode_info).pack())
-        buf.write("")
         buf.write(xcffib.pack_list(name, "c"))
         return self.send_request(16, buf, CreateModeCookie, is_checked=is_checked)
     def DestroyMode(self, mode, is_checked=False):
@@ -789,7 +804,6 @@ class randrExtension(xcffib.Extension):
     def SetCrtcConfig(self, crtc, timestamp, config_timestamp, x, y, mode, rotation, outputs_len, outputs, is_checked=True):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xIIIhhIH2x", crtc, timestamp, config_timestamp, x, y, mode, rotation))
-        buf.write("")
         buf.write(xcffib.pack_list(outputs, "I"))
         return self.send_request(21, buf, SetCrtcConfigCookie, is_checked=is_checked)
     def GetCrtcGammaSize(self, crtc, is_checked=True):
@@ -816,13 +830,12 @@ class randrExtension(xcffib.Extension):
         buf.write(struct.pack("=xx2xIH2x", crtc, filter_len))
         buf.write(transform.pack() if hasattr(transform, "pack") else TRANSFORM.synthetic(*transform).pack())
         buf.write(xcffib.pack_list(filter_name, "c"))
-        buf.write("")
         buf.write(xcffib.pack_list(filter_params, "i"))
         return self.send_request(26, buf, is_checked=is_checked)
-    def GetCrtcTransform(self, crtc, is_checked=True):
+    def GetCrtcTransform(self, crtc, is_checked=False):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xI", crtc))
-        return self.send_request(27, buf, GetCrtcTransformCookie, is_checked=is_checked)
+        return self.send_request(27, buf, is_checked=is_checked)
     def GetPanning(self, crtc, is_checked=True):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xI", crtc))
@@ -866,7 +879,6 @@ class randrExtension(xcffib.Extension):
     def ConfigureProviderProperty(self, provider, property, pending, range, values_len, values, is_checked=False):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xIIBB2x", provider, property, pending, range))
-        buf.write("")
         buf.write(xcffib.pack_list(values, "i"))
         return self.send_request(38, buf, is_checked=is_checked)
     def ChangeProviderProperty(self, provider, property, type, format, mode, num_items, data, is_checked=False):
@@ -882,4 +894,17 @@ class randrExtension(xcffib.Extension):
         buf = six.BytesIO()
         buf.write(struct.pack("=xx2xIIIIIBB2x", provider, property, type, long_offset, long_length, delete, pending))
         return self.send_request(41, buf, GetProviderPropertyCookie, is_checked=is_checked)
+    def GetMonitors(self, window, get_active, is_checked=True):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xIB", window, get_active))
+        return self.send_request(42, buf, GetMonitorsCookie, is_checked=is_checked)
+    def SetMonitor(self, window, monitorinfo, is_checked=False):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xI", window))
+        buf.write(monitorinfo.pack() if hasattr(monitorinfo, "pack") else MonitorInfo.synthetic(*monitorinfo).pack())
+        return self.send_request(43, buf, is_checked=is_checked)
+    def DeleteMonitor(self, window, name, is_checked=False):
+        buf = six.BytesIO()
+        buf.write(struct.pack("=xx2xII", window, name))
+        return self.send_request(44, buf, is_checked=is_checked)
 xcffib._add_ext(key, randrExtension, _events, _errors)
